import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;
import java.util.*;

public class Check implements Checker {

    public static char[][][] FIGURES = {
            {
                    "###".toCharArray()
            },
            {
                    "##".toCharArray()
            },
            {
                    "#.".toCharArray(),
                    "##".toCharArray()
            }
    };

    static char[][] deepClone(char[][] a) {
        char[][] res = new char[a.length][];
        for (int i = 0; i < res.length; i++) {
            res[i] = a[i].clone();
        }
        return res;
    }

    static class Field {
        final char[][] f;

        Field(int x, int y) {
            f = new char[y][x];
            for (char[] fx : f) {
                Arrays.fill(fx, '.');
            }
        }

        Field(char[][] f) {
            this.f = f;
        }

        Field update(char[][] figure, int pos) {
            int h = f.length - 1;
            char[][] f = deepClone(this.f);
            outer: for (; h >= figure.length - 1; h--) {
                for (int i = 0; i < figure.length; i++) {
                    for (int j = 0; j < figure[i].length; j++) {
                        if (figure[i][j] == '#' && f[h - i][pos + j] == '#') {
                            break outer;
                        }
                    }
                }
            }
            h++;
            for (int i = 0; i < figure.length; i++) {
                for (int j = 0; j < figure[i].length; j++) {
                    if (figure[i][j] == '#') {
                        f[h - i][pos + j] = '#';
                    }
                }
            }

            for (int i = 0; i < f.length; i++) {
                boolean all = true;
                for (int j = 0; j < f[i].length; j++) {
                    all &= f[i][j] == '#';
                }
                if (all) {
                    for (int k = i + 1; k < f.length; k++) {
                        f[k - 1] = f[k].clone();
                    }
                    i--;
                }
            }
            return new Field(f);
        }

        int lines() {
            int res = 0;
            for (char[] fx : f) {
                for (char x : fx) {
                	if (x == '#') {
                		res++;
                		break;
                	}
                }
            }
            return res;
        }

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            for (int i = f.length - 1; i >= 0; i--) {
                sb.append(new String(f[i])).append("\n");
            }
            return sb.toString();
        }
    }

    static char[][] rotate(char[][] fig) {
        int n = fig.length;
        int m = fig[0].length;
        char[][] res = new char[m][n];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                res[m - 1 - j][i] = fig[i][j];
            }
        }
        return res;
    }

	public Outcome test(InStream inf, InStream ouf, InStream ans) {
		int n = inf.nextInt();
		int[] types = new int[n];
		for (int i = 0; i < n; i++) {
			types[i] = inf.nextInt();
		}

		int[] rot = new int[n];
		int[] pos = new int[n];
		for (int i = 0; i < n; i++) {
			pos[i] = ouf.nextInt();
			rot[i] = ouf.nextInt();
		}
        Field f = new Field(3, 8);
        for (int i = 0; i < n; i++) {
            if (!(0 <= rot[i] && rot[i] < 4)) {
            	return new Outcome(Outcome.Type.WA, 
            		"Rotates must be int from 0 to 3, " + rot[i] + " found");
            }
            char[][] fig = FIGURES[types[i] - 1];
            for (int j = 0; j < rot[i]; j++) {
                fig = rotate(fig);
            }
            try {
                f = f.update(fig, pos[i]);
            } catch (IndexOutOfBoundsException e) {
            	return new Outcome(Outcome.Type.WA, 
            		"Bad position for figure " + types[i] + " : " + pos[i]);
            }

            //System.out.println(i);
	    //System.out.println(f.toString());

            if (f.lines() > 4) {
            	return new Outcome(Outcome.Type.WA, 
            		"Game over! After " + (i + 1) + " turns.");
            }
        }
    	return new Outcome(Outcome.Type.OK, 
    		"Win!");
	}
}
